"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const DiscoveryService_1 = require("./src/DiscoveryService");
const BasicSystemRepository_1 = __importDefault(require("./src/persistence/BasicSystemRepository"));
const ConnectionService_1 = require("./src/ConnectionService");
const Router_1 = __importDefault(require("./src/Router"));
const DeviceContext_1 = require("./src/device/DeviceContext");
const config_1 = __importDefault(require("./config"));
const ip_1 = __importDefault(require("ip"));
const fs_1 = __importDefault(require("fs"));
const RemoteService_1 = require("./src/RemoteService");
if (process.env.DISCOVERY_KEYWORD) {
    config_1.default.discoveryKeyword = process.env.DISCOVERY_KEYWORD;
}
if (process.env.DISCOVERY_INTERVAL) {
    config_1.default.discoveryInterval = Number.parseInt(process.env.DISCOVERY_INTERVAL);
}
if (process.env.DISCOVERY_PORT) {
    config_1.default.discoveryPort = Number.parseInt(process.env.DISCOVERY_PORT);
}
if (process.env.CONNECTION_PORT) {
    config_1.default.connectionPort = Number.parseInt(process.env.CONNECTION_PORT);
}
if (process.env.AUTHENTICATED_PORT) {
    config_1.default.authenticatedPort = Number.parseInt(process.env.AUTHENTICATED_PORT);
}
if (process.env.BROADCASTING_PORT) {
    config_1.default.broadcastingPort = Number.parseInt(process.env.BROADCASTING_PORT);
}
if (process.env.ENABLE_DISCOVERY) {
    config_1.default.enableDiscovery = process.env.ENABLE_DISCOVERY.toLowerCase() === 'true';
}
const initServer = () => {
    const app = (0, express_1.default)();
    app.get('/', (req, res) => {
        res.redirect('/ui/index.html');
    });
    app.use('/ui', express_1.default.static(__dirname + '/../out'));
    const server = app.listen(config_1.default.connectionPort);
    (0, ConnectionService_1.initConnectionService)(server, false);
};
const serverIp = ip_1.default.address();
if (serverIp) {
    const serverAddress = 'http://' + serverIp + ':' + ((_a = config_1.default.connectionPort) !== null && _a !== void 0 ? _a : config_1.default.authenticatedPort);
    fs_1.default.writeFile('ServerLink.txt', serverAddress, (err) => {
        if (err) {
            console.log('On saving server link', err);
        }
    });
    console.log('Server running on ' + serverAddress);
}
Router_1.default.systemRepository = (0, BasicSystemRepository_1.default)();
(0, DeviceContext_1.initDeviceContext)().then(() => {
    if (config_1.default.enableDiscovery) {
        (0, DiscoveryService_1.initDiscovery)();
    }
    if (config_1.default.enableInternalAccess && config_1.default.connectionPort) {
        initServer();
    }
    if (config_1.default.enableExternalAccess && config_1.default.authenticatedPort) {
        (0, RemoteService_1.initRemote)();
    }
});
