"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stopRemote = exports.authenticate = exports.initRemote = exports.setRemoteCredentials = exports.getRemoteAccessState = void 0;
const fs_1 = __importDefault(require("fs"));
const config_1 = __importDefault(require("../config"));
const generate_password_1 = __importDefault(require("generate-password"));
const selfsigned_1 = __importDefault(require("selfsigned"));
const crypto_1 = __importDefault(require("crypto"));
const https_1 = __importDefault(require("https"));
const express_1 = __importDefault(require("express"));
const cookie_parser_1 = __importDefault(require("cookie-parser"));
const body_parser_1 = __importDefault(require("body-parser"));
const cors_1 = __importDefault(require("cors"));
const ConnectionService_1 = require("./ConnectionService");
const ControllerContext_1 = __importDefault(require("./controller/ControllerContext"));
const apiUrl = 'https://remote.stargate-system.com';
const connections = new Map();
const clients = new Map();
const states = {
    on: 'on',
    off: 'off',
    init: 'init',
    registered: 'ready',
    error: 'error'
};
let state = states.off;
let serverId;
let serverKey;
let server;
let currentIp;
let registerTimeout;
let ipCheckInterval;
let initTimeout;
let stateTimeout;
const getRemoteAccessState = () => state;
exports.getRemoteAccessState = getRemoteAccessState;
const setRemoteCredentials = (key, pass) => {
    (0, exports.stopRemote)();
    try {
        if (!fs_1.default.existsSync('remote')) {
            fs_1.default.mkdirSync('remote');
        }
        fs_1.default.writeFileSync('remote/id.txt', `${key}, ${pass}`);
    }
    catch (err) {
        console.log('On saving remote credentials', err);
    }
    (0, exports.initRemote)();
};
exports.setRemoteCredentials = setRemoteCredentials;
const initRemote = () => __awaiter(void 0, void 0, void 0, function* () {
    if (!initTimeout && (state === states.off || state === states.error)) {
        config_1.default.enableExternalAccess = true;
        try {
            const idFile = fs_1.default.readFileSync('remote/id.txt').toString();
            const [id, key] = idFile.split(',').map((word) => word.trim());
            serverId = id;
            serverKey = key;
            if (!id || !key) {
                throw new Error('Id or key missing');
            }
            changeState(states.init);
            currentIp = yield getPublicIp();
            if (currentIp) {
                initServer(currentIp);
                initIpCheck();
                changeState(states.on);
            }
            else {
                initTimeout = setTimeout(() => {
                    changeState(states.off);
                    initTimeout = undefined;
                    (0, exports.initRemote)();
                }, 5000);
            }
        }
        catch (err) {
            changeState(states.error);
            console.log('Unable to initialize remote service:', (err instanceof Error && err.message.match(/no such file or directory/i)) ? 'No remote credentials available' : err);
        }
    }
});
exports.initRemote = initRemote;
const authenticate = (auth) => {
    if (auth && auth.clientId && clients.get(auth.clientId) === auth.clientKey) {
        return true;
    }
    return false;
};
exports.authenticate = authenticate;
const stopRemote = (error = false) => {
    config_1.default.enableExternalAccess = false;
    if (registerTimeout) {
        clearTimeout(registerTimeout);
        registerTimeout = undefined;
    }
    if (ipCheckInterval) {
        clearInterval(ipCheckInterval);
        ipCheckInterval = undefined;
    }
    if (initTimeout) {
        clearTimeout(initTimeout);
        initTimeout = undefined;
    }
    connections.clear();
    clients.clear();
    if (server) {
        server.close((err) => {
            if (err) {
                console.log('On closing https server', err);
            }
        });
        server = undefined;
    }
    changeState(error ? states.error : states.off);
};
exports.stopRemote = stopRemote;
const changeState = (newState) => {
    state = newState;
    if (!stateTimeout) {
        stateTimeout = setTimeout(() => {
            stateTimeout = undefined;
            // TODO move remoteAccess to EventNames
            ControllerContext_1.default.forwardServerEvent('remoteAccess', [state]);
        }, 100);
    }
};
const initIpCheck = () => {
    if (ipCheckInterval) {
        clearInterval(ipCheckInterval);
    }
    ipCheckInterval = setInterval(() => __awaiter(void 0, void 0, void 0, function* () {
        const ip = yield getPublicIp();
        if (ip && ip !== currentIp) {
            currentIp = ip;
            initServer(ip);
        }
    }), 60000);
};
const initServer = (ip) => {
    if (server) {
        server.close((err) => {
            if (err) {
                console.log('On closing https server', err);
            }
        });
    }
    if (registerTimeout) {
        clearTimeout(registerTimeout);
        registerTimeout = undefined;
    }
    register();
    const app = (0, express_1.default)();
    app.use((0, cookie_parser_1.default)());
    app.use(body_parser_1.default.json());
    app.post('/connect', (0, cors_1.default)({ origin: apiUrl }), connect);
    app.get('/login', login);
    app.use('/ui', (req, res, next) => {
        var _a;
        try {
            if (((_a = req.cookies) === null || _a === void 0 ? void 0 : _a.stargate_client) && (0, exports.authenticate)(JSON.parse(req.cookies.stargate_client))) {
                next();
            }
            else {
                res.status(403).end();
            }
        }
        catch (err) {
            console.log('On static content authentication', err);
            res.status(403).end();
        }
    });
    app.use('/ui', express_1.default.static(__dirname + '/../../out'));
    server = https_1.default.createServer(getCertificates(ip), app).listen(config_1.default.authenticatedPort);
    (0, ConnectionService_1.initConnectionService)(server, true);
};
const getPublicIp = () => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const ipResponse = yield fetch('https://api.ipify.org?format=json');
        if (ipResponse.ok) {
            const ip = yield ipResponse.json();
            return ip.ip;
        }
    }
    catch (_a) { }
    return undefined;
});
const getCertificates = (ip) => {
    const generated = selfsigned_1.default.generate([{ name: 'commonName', value: ip }], { keySize: 2048, days: 365 });
    return { key: generated.private, cert: generated.cert };
};
const register = () => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    if (registerTimeout) {
        clearTimeout(registerTimeout);
        registerTimeout = undefined;
    }
    if (serverId && serverKey) {
        let registerResponse;
        try {
            registerResponse = yield fetch(`${apiUrl}/api/register`, {
                method: 'POST',
                body: JSON.stringify({ id: serverId, key: serverKey, port: config_1.default.authenticatedPort })
            });
        }
        catch (_b) { }
        if (registerResponse && registerResponse.ok) {
            console.log('Registered in central service');
            changeState(states.registered);
        }
        else {
            if (!registerResponse || registerResponse.status !== 403) {
                registerTimeout = setTimeout(register, 10000);
                console.log('Failed to register in central service', (_a = registerResponse === null || registerResponse === void 0 ? void 0 : registerResponse.status) !== null && _a !== void 0 ? _a : 'fetch failed');
            }
            else {
                console.log('Failed to register in central service: wrong server credentials');
                (0, exports.stopRemote)(true);
            }
        }
    }
});
const connect = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const { connectionId } = req.body;
    let response;
    try {
        response = yield fetch(`${apiUrl}/api/connect`, { method: 'POST', body: JSON.stringify({ connectionId, serverId, serverKey }) });
    }
    catch (_a) { }
    if (response && response.ok) {
        const keys = yield response.json();
        connections.set(connectionId, keys.connectionKey);
        setTimeout(() => connections.delete(connectionId), 20000);
        res.json({ verificationKey: keys.verificationKey });
    }
    else {
        res.status(403).end();
    }
});
const login = (req, res) => {
    const id = req.query.id;
    const key = req.query.key;
    if (typeof id !== 'string' || typeof key !== 'string' || connections.get(id) !== key) {
        res.status(403).end();
        return;
    }
    connections.delete(id);
    const clientId = crypto_1.default.randomUUID();
    const clientKey = generate_password_1.default.generate({
        length: 32,
        numbers: true
    });
    clients.set(clientId, clientKey);
    // 1 hour
    const maxAge = 3600000;
    setTimeout(() => clients.delete(clientId), maxAge);
    res.cookie('stargate_client', JSON.stringify({ clientId, clientKey }), { maxAge });
    res.redirect('/ui/index.html');
};
