"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseControllerConnector = void 0;
const core_1 = require("@stargate-system/core");
const ServerStorageRequestListener_1 = require("../common/ServerStorageRequestListener");
class BaseControllerConnector {
    constructor(connection) {
        this.id = '';
        this.onDisconnect = () => { };
        this.onSubscribed = () => { };
        this.onUnsubscribed = () => { };
        this.onValueMessage = () => { };
        this.onDeviceEvent = () => { };
        this.onPipeEvent = () => { };
        this.sendDeviceEvent = (event, data) => {
            this._connection.functionalHandler.sendCommand(core_1.Keywords.deviceEvent, [event, ...data]);
        };
        this.sendPipeEvent = (event, data) => {
            this._connection.functionalHandler.sendCommand(core_1.Keywords.pipeEvent, [event, ...data]);
        };
        this.sendServerEvent = () => { };
        this.sendJoinData = (systemImage, connectedDevices) => {
            this._connection.functionalHandler.sendCommand(core_1.Keywords.joinData, [JSON.stringify(systemImage), ...connectedDevices]);
        };
        this.sendValueMessage = (valueMessage) => {
            valueMessage.forEach((value) => this._connection.sendValue(value));
        };
        this._connection = connection;
        this._connection.addStateChangeListener((state) => {
            if (state === core_1.ConnectionState.closed) {
                this.onDisconnect();
            }
        });
        this._connection.onValueMessage = (msg) => this.onValueMessage(msg);
        const functionalHandler = this._connection.functionalHandler;
        functionalHandler.addCommandListener(core_1.Keywords.subscribe, (params) => {
            this.onSubscribed(params !== null && params !== void 0 ? params : []);
        });
        functionalHandler.addCommandListener(core_1.Keywords.unsubscribe, (params) => {
            this.onUnsubscribed(params !== null && params !== void 0 ? params : []);
        });
        functionalHandler.addCommandListener(core_1.Keywords.deviceEvent, (params) => {
            if (params && params[0]) {
                const eventName = params[0];
                this.onDeviceEvent(eventName, params.slice(1));
            }
        });
        functionalHandler.addCommandListener(core_1.Keywords.pipeEvent, (params) => {
            if (params && params[0]) {
                const eventName = params[0];
                this.onPipeEvent(eventName, params.slice(1));
            }
        });
        (0, ServerStorageRequestListener_1.setServerStorageRequestListeners)(this._connection, 'controller');
        this._connection.setReady();
    }
}
exports.BaseControllerConnector = BaseControllerConnector;
