"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const device_1 = require("@stargate-system/device");
const settings_1 = __importDefault(require("../utils/settings"));
let camera;
let buffer;
let video;
let useGain;
let gain;
const init = () => {
    const { camera } = settings_1.default.getSettings();
    useGain = device_1.GateDevice.factory.createBoolean(device_1.Directions.input);
    useGain.valueName = 'Use gain';
    useGain.visibility = device_1.ValueVisibility.settings;
    useGain.setValue(camera.useGain);
    useGain.onRemoteUpdate = () => {
        if (useGain.value !== undefined) {
            camera.useGain = useGain.value;
            settings_1.default.save();
        }
    };
    gain = device_1.GateDevice.factory.createFloat(device_1.Directions.input);
    gain.valueName = 'Gain';
    gain.visibility = device_1.ValueVisibility.settings;
    gain.setRange([0.1, 20]);
    gain.setValue(camera.gain);
    gain.onRemoteUpdate = () => {
        if (gain.value !== undefined) {
            camera.gain = gain.value;
            settings_1.default.save();
        }
    };
    video = device_1.GateDevice.factory.createString(device_1.Directions.output);
    video.valueName = "Camera";
    video.visibility = device_1.ValueVisibility.hidden;
    video.onSubscriptionChange = (subscribed) => {
        if (subscribed) {
            startCamera();
        }
        else {
            stopCamera();
        }
    };
};
const basicSettings = [
    '-t', '0',
    '-o', '-',
    '--width', '640',
    '--height', '480',
    '--framerate', '30',
    '--codec', 'mjpeg',
    '-n'
];
const getSettings = () => {
    var _a, _b;
    const settings = [...basicSettings];
    if (useGain.value) {
        settings.push('--gain');
        settings.push((_b = (_a = gain.value) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : '1');
    }
    return settings;
};
const startCamera = () => {
    if (!camera) {
        camera = (0, child_process_1.spawn)('rpicam-vid', getSettings());
        camera.stdout.on('data', handleData);
    }
};
const handleData = (chunk) => {
    const frameStartIndex = getFrameStart(chunk);
    if (buffer === undefined) {
        if (!frameStartIndex || frameStartIndex === 0) {
            buffer = chunk;
        }
        else {
            buffer = chunk.subarray(0, frameStartIndex);
            pushBuffer();
            buffer = chunk.subarray(frameStartIndex);
        }
    }
    else {
        if (!frameStartIndex) {
            buffer = Buffer.concat([buffer, chunk]);
        }
        else if (frameStartIndex === 0) {
            pushBuffer();
            buffer = chunk;
        }
        else {
            buffer = Buffer.concat([buffer, chunk.subarray(0, frameStartIndex)]);
            pushBuffer();
            buffer = chunk.subarray(frameStartIndex);
        }
    }
};
const getFrameStart = (chunk) => {
    for (let i = 0; i < chunk.length; i++) {
        if (chunk[i] === 0xFF) {
            if (chunk[i + 1] === 0xD8) {
                return i;
            }
        }
    }
    return undefined;
};
const pushBuffer = () => {
    if (buffer) {
        video.setValue(buffer.toString('base64'));
        buffer = undefined;
    }
};
const stopCamera = () => {
    if (camera) {
        camera.kill();
        camera = undefined;
        buffer = undefined;
        video.setValue(undefined);
    }
};
const cameraApi = {
    init
};
exports.default = cameraApi;
