"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const device_1 = require("@stargate-system/device");
const core_1 = require("@stargate-system/core");
const motor_1 = require("../utils/motor");
const settings_1 = __importDefault(require("../utils/settings"));
let leftMotor;
let rightMotor;
let chassisCommand;
let inputTimeout;
const init = () => {
    const { chassis } = settings_1.default.getSettings();
    leftMotor = new motor_1.Motor(chassis.leftMotorPins[0], chassis.leftMotorPins[1]);
    rightMotor = new motor_1.Motor(chassis.rightMotorPins[0], chassis.rightMotorPins[1]);
    chassisCommand = device_1.GateDevice.factory.createString(device_1.Directions.input);
    chassisCommand.valueName = 'Chassis command';
    chassisCommand.visibility = core_1.ValueVisibility.hidden;
    chassisCommand.onRemoteUpdate = handleMotors;
};
const resetTimeout = () => {
    if (inputTimeout) {
        clearTimeout(inputTimeout);
    }
    inputTimeout = setTimeout(() => stop(), 500);
};
const handleMotors = (wasChanged) => {
    var _a;
    resetTimeout();
    if (wasChanged) {
        const command = (_a = chassisCommand.value) === null || _a === void 0 ? void 0 : _a.split(':').map((value) => Number.parseFloat(value));
        if (command) {
            setMotors(command);
        }
    }
};
const setMotors = (command) => {
    const [forwardReverse, leftRight] = command;
    const motorCommand = [forwardReverse + leftRight, forwardReverse - leftRight];
    motorCommand[0] = normalize(motorCommand[0]);
    motorCommand[1] = normalize(motorCommand[1]);
    leftMotor.run(motorCommand[0]);
    rightMotor.run(motorCommand[1]);
};
const stop = () => {
    leftMotor.stop();
    rightMotor.stop();
};
const normalize = (input) => {
    if (input > 1) {
        return 1;
    }
    if (input < -1) {
        return -1;
    }
    return input;
};
const chassisApi = {
    init
};
exports.default = chassisApi;
