"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const device_1 = require("@stargate-system/device");
const servo_1 = require("../utils/servo");
const settings_1 = __importDefault(require("../utils/settings"));
const init = () => {
    const { head } = settings_1.default.getSettings();
    const centerX = device_1.GateDevice.factory.createFloat(device_1.Directions.input);
    centerX.valueName = 'Head center X';
    centerX.visibility = device_1.ValueVisibility.settings;
    centerX.setRange([0, 1]);
    centerX.setValue(head.horizontalCenter);
    centerX.onRemoteUpdate = () => {
        if (centerX.value !== undefined) {
            servoX.moveTo(centerX.value);
            head.horizontalCenter = centerX.value;
            settings_1.default.save();
        }
    };
    const centerY = device_1.GateDevice.factory.createFloat(device_1.Directions.input);
    centerY.valueName = 'Head center Y';
    centerY.visibility = device_1.ValueVisibility.settings;
    centerY.setRange([0, 1]);
    centerY.setValue(head.verticalCenter);
    centerY.onRemoteUpdate = () => {
        if (centerY.value !== undefined) {
            servoY.moveTo(centerY.value);
            head.verticalCenter = centerY.value;
            settings_1.default.save();
        }
    };
    const horizontal = device_1.GateDevice.factory.createFloat(device_1.Directions.input);
    horizontal.valueName = 'Head X';
    horizontal.visibility = device_1.ValueVisibility.hidden;
    horizontal.setRange([0, 1]);
    horizontal.setValue(centerX.value);
    const vertical = device_1.GateDevice.factory.createFloat(device_1.Directions.input);
    vertical.valueName = 'Head Y';
    vertical.visibility = device_1.ValueVisibility.hidden;
    vertical.setRange([0, 1]);
    vertical.setValue(centerY.value);
    const servoX = new servo_1.Servo(head.horizontalServoPin, { initialPosition: centerX.value });
    horizontal.onRemoteUpdate = () => {
        if (horizontal.value !== undefined) {
            servoX.moveTo(horizontal.value);
        }
    };
    const servoY = new servo_1.Servo(head.verticalServoPin, { initialPosition: centerY.value });
    vertical.onRemoteUpdate = () => {
        if (vertical.value !== undefined) {
            servoY.moveTo(vertical.value);
        }
    };
};
const headApi = {
    init
};
exports.default = headApi;
